# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
# If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
# © 2025 cswimr

from typing import Any, Literal

from typing_extensions import override


class _MissingSentinel:
    __slots__ = ()

    @override
    def __eq__(self, other) -> bool:
        return False

    def __bool__(self) -> bool:
        return False

    @override
    def __hash__(self) -> int:
        return 0

    @override
    def __repr__(self) -> Literal["..."]:
        return "..."


MISSING: Any = _MissingSentinel()



# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
# If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
# © 2025 cswimr

from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel, Field


class Debug(BaseModel):
    ping: bool
    query: bool
    bedrock: bool
    srv: bool
    query_mismatch: bool = Field(alias="querymismatch")
    ip_in_srv: bool = Field(alias="ipinsrv")
    cname_in_srv: bool = Field(alias="cnameinsrv")
    animated_motd: bool = Field(alias="animatedmotd")
    cache_hit: bool = Field(alias="cachehit")
    cache_time: datetime = Field(alias="cachetime")
    cache_expire: datetime = Field(alias="cacheexpire")
    api_version: int = Field(alias="apiversion")


class Protocol(BaseModel):
    version: int
    name: Optional[str] = None


class Object(BaseModel):
    raw: str
    clean: str
    html: str


class ListObject(BaseModel):
    raw: List[str]
    clean: List[str]
    html: List[str]


class Player(BaseModel):
    name: str
    uuid: str


class Players(BaseModel):
    online: int
    max: int
    players: Optional[List[Player]] = None


class Mod(BaseModel):
    name: str
    version: str


class ServerStatus(BaseModel):
    """ServerStatus class"""

    online: bool
    debug: Debug
    ip: Optional[str] = None
    port: Optional[int] = None
    hostname: Optional[str] = None
    version: Optional[str] = None
    protocol: Optional[Protocol] = None
    icon: Optional[str] = None
    software: Optional[str] = None
    map: Optional[Object] = None
    gamemode: Optional[str] = None
    server_id: Optional[str] = Field(default=None, alias="serverid")
    eula_blocked: Optional[bool] = None
    motd: Optional[ListObject] = None
    players: Optional[Players] = None
    plugins: Optional[List[Mod]] = None
    mods: Optional[List[Mod]] = None
    info: Optional[ListObject] = None

    def is_bedrock(self) -> bool:
        if self.server_id or self.gamemode:
            return True
        return False

    def is_java(self) -> bool:
        return not self.is_bedrock()



# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
# If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
# © 2025 cswimr

from sys import version as pyversion
from typing import Dict, Optional

import aiohttp

from pymcsrvstatus.httpurl import HttpUrl
from pymcsrvstatus.models import ServerStatus
from pymcsrvstatus.version import __version__

_URL = HttpUrl("https://api.mcsrvstat.us")
_API_VERSION = "3"


def _headers(extra_user_agent: Optional[str]) -> Dict[str, str]:
    return {"User-Agent": f"pymcsrvstatus/{__version__}{f' {extra_user_agent}' if extra_user_agent else ''} (Python {pyversion})".replace("\n", "")}


async def fetch(address: str, extra_user_agent: Optional[str] = None) -> ServerStatus:
    url = str(_URL / _API_VERSION / address)
    async with aiohttp.ClientSession(headers=_headers(extra_user_agent)) as session, session.get(url) as response:
        return ServerStatus(**await response.json())


async def check_if_online(address: str, extra_user_agent: Optional[str] = None) -> bool:
    url = str(_URL / "simple" / address)
    async with aiohttp.ClientSession(headers=_headers(extra_user_agent)) as session, session.get(url) as response:
        if response.status == 200:
            return True
        return False


def get_icon_url(address: str) -> HttpUrl:
    return _URL / "icon" / address



# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
# If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
# © 2025 cswimr

from .api import check_if_online, fetch, get_icon_url
from .models import ServerStatus

__all__ = ["check_if_online", "fetch", "get_icon_url", "ServerStatus"]



# This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
# If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
# © 2025 cswimr

from typing import Optional

from pydantic import HttpUrl as PydanticHttpUrl
from typing_extensions import Self

from pymcsrvstatus.missing import MISSING


class HttpUrl(PydanticHttpUrl):
    """
    A subclass of `pydantic.HttpUrl` that adds some useful methods and operator overloads for more ergonomic path manipulation and URL composition.

    Operations:
      - `url / "segment"`  - returns a new HttpUrl with "segment" appended to the path
      - `url /= "segment"` - in-place append of "segment" to the path

    Examples:
        >>> u = HttpUrl("https://example.com/api")
        >>> u2 = u / "v1" / "users"
        >>> print(u2)
        https://example.com/api/v1/users

        >>> u2 /= "123"
        >>> print(u2)
        https://example.com/api/v1/users/123
    """

    def __truediv__(self, other: str) -> Self:
        return self.join(other)

    def __itruediv__(self, other: str) -> Self:
        return self.__truediv__(other)

    @property
    def base(self) -> Self:
        """Wrapper around `HttpUrl.join()` that returns the base URL (scheme, host, etc.).

        Returns:
            Self: The new HttpUrl object pointing to the base URL of the source HttpUrl object.
        """
        return self.join(None, query=None, fragment=None)

    def join(self, path: Optional[str] = MISSING, /, *, query: Optional[str] = MISSING, fragment: Optional[str] = MISSING) -> Self:
        """Create a new HttpUrl object using `HttpUrl.build()`.

        Args:
            path (str): The path to add to this HttpUrl's path.
                Defaults to MISSING, will replace the original path if set to `None`.
            query (str | None): The query to replace this HttpUrl's query with.
                Defaults to MISSING, will replace the original query if set to `None`.
            fragment (str | None): The fragment to replace this HttpUrl's fragment with.
                Defaults to MISSING, will replace the original fragment if set to `None`.

        Returns:
            Self: The new HttpUrl object.
        """

        def _strip(s: str, /) -> str:
            return s.lstrip("/").rstrip("/")

        base = _strip(self.path or "")

        if path is MISSING:
            new_path = base
        elif path is None:
            new_path = ""
        else:
            seg = _strip(path)
            new_path = f"{base}/{seg}" if base else seg

        return self.build(
            scheme=self.scheme,
            username=self.username,
            password=self.password,
            host=self.host or "",
            port=self.port,
            path=new_path,
            query=self.query if query is MISSING else query,
            fragment=self.fragment if fragment is MISSING else fragment,
        )
